"""
Set of colour schemes to use when creating cloud
"""
red = ['lightcoral','salmon','darksalmon','lightsalmon','crimson','red','firebrick','darkred','indianred']
blue = ['lightblue','skyblue','lightskyblue','cornflowerblue','mediumslateblue','royalblue','mediumblue','darkblue','navy','midnightblue']
green = ['palegreen','lightgreen','mediumspringgreen','springgreen','mediumseagreen','seagreen','forestgreen','green','darkgreen']
pink = ['pink','lightpink','hotpink','deeppink','mediumvioletred','palevioletred']
brown = ['sandybrown','goldenrod','darkgoldenrod','peru','chocolate','saddlebrown','sienna','brown','maroon']
white = ['snow','honeydew','mintcream','azure','aliceblue','ghostwhite','whitesmoke','seashell','beige','oldlace','floralwhite','ivory','antiquewhite','linen','lavenderblush','mistyrose']
gray = ['gainsboro','lightgray','silver','darkgray','gray','dimgray','lightslategray','slategray','darkslategray']
orange = ['coral','tomato','lightsalmon','orangered','darkorange','orange']
yellow = ['lightgoldenrodyellow','papayawhip','moccasin','peachpuff','palegoldenrod','khaki','darkkhaki']
multi = ['firebrick', 'darkgreen', 'midnightblue','darkorange','mediumpurple']


from csv import reader
from io import open

stoplist=["a","able","about","above","according","accordingly","across","actually","after","afterwards","again","against","ain't","all","allow","allows","almost","alone","along","already","also","although","always","am","among","amongst","an","and","another","any","anybody","anyhow","anyone","anything","anyway","anyways","anywhere","apart","appear","appreciate","appropriate","are","aren't","around","as","a's","aside","ask","asking","associated","at","available","away","awfully","be","became","because","become","becomes","becoming","been","before","beforehand","behind","being","believe","below","beside","besides","best","better","between","beyond","both","brief","but","by","came","can","cannot","cant","can't","cause","causes","certain","certainly","changes","clearly","c'mon","co","com","come","comes","concerning","consequently","consider","considering","contain","containing","contains","corresponding","could","couldn't","course","c's","currently","definitely","described","despite","did","didn't","different","do","does","doesn't","doing","done","don't","down","downwards","during","each","edu","eg","eight","either","else","elsewhere","enough","entirely","especially","et","etc","even","ever","every","everybody","everyone","everything","everywhere","ex","exactly","example","except","far","few","fifth","first","five","followed","following","follows","for","former","formerly","forth","four","from","further","furthermore","get","gets","getting","given","gives","go","goes","going","gone","got","gotten","greetings","had","hadn't","happens","hardly","has","hasn't","have","haven't","having","he","hello","help","hence","her","here","hereafter","hereby","herein","here's","hereupon","hers","herself","he's","hi","him","himself","his","hither","hopefully","how","howbeit","however","i","i'd","ie","if","ignored","i'll","i'm","immediate","in","inasmuch","inc","indeed","indicate","indicated","indicates","inner","insofar","instead","into","inward","is","isn't","it","it'd","it'll","its","it's","itself","i've","just","keep","keeps","kept","know","known","knows","last","lately","later","latter","latterly","least","less","lest","let","let's","like","liked","likely","little","look","looking","looks","ltd","mainly","many","may","maybe","me","mean","meanwhile","merely","might","more","moreover","most","mostly","much","must","my","myself","name","namely","nd","near","nearly","necessary","need","needs","neither","never","nevertheless","new","next","nine","no","nobody","non","none","noone","nor","normally","not","nothing","novel","now","nowhere","obviously","of","off","often","oh","ok","okay","old","on","once","one","ones","only","onto","or","other","others","otherwise","ought","our","ours","ourselves","out","outside","over","overall","own","particular","particularly","per","perhaps","placed","please","plus","possible","presumably","probably","provides","que","quite","qv","rather","rd","re","really","reasonably","regarding","regardless","regards","relatively","respectively","right","said","same","saw","say","saying","says","second","secondly","see","seeing","seem","seemed","seeming","seems","seen","self","selves","sensible","sent","serious","seriously","seven","several","shall","she","should","shouldn't","since","six","so","some","somebody","somehow","someone","something","sometime","sometimes","somewhat","somewhere","soon","sorry","specified","specify","specifying","still","sub","such","sup","sure","take","taken","tell","tends","th","than","thank","thanks","thanx","that","thats","that's","the","their","theirs","them","themselves","then","thence","there","thereafter","thereby","therefore","therein","theres","there's","thereupon","these","they","they'd","they'll","they're","they've","think","third","this","thorough","thoroughly","those","though","three","through","throughout","thru","thus","to","together","too","took","toward","towards","tried","tries","truly","try","trying","t's","twice","two","un","under","unfortunately","unless","unlikely","until","unto","up","upon","us","use","used","useful","uses","using","usually","value","various","very","via","viz","vs","want","wants","was","wasn't","way","we","we'd","welcome","well","we'll","went","were","we're","weren't","we've","what","whatever","what's","when","whence","whenever","where","whereafter","whereas","whereby","wherein","where's","whereupon","wherever","whether","which","while","whither","who","whoever","whole","whom","who's","whose","why","will","willing","wish","with","within","without","wonder","won't","would","wouldn't","yes","yet","you","you'd","you'll","your","you're","yours","yourself","yourselves","you've","zero","â","int","'em","os","ext",'nigger','nigga','niggas','niggers','niggerz','niggaz']

def load_txt(file, 
             minwordlength=3, 
             extra_stopwords=[]):
    """
    Load text file and count words
    
    Arguments:
    file -- file name of text file to load
    
    Returns:
    Word count dictionary { word: count }
    """
    try:
        with open(file,"r") as f:
            text = f.read().lower()
            wordcount = count_words(text, minwordlength=minwordlength, extra_stopwords=extra_stopwords)
        return wordcount
    except IOError as error:
        print (error)
        return None

def count_words(text, 
                minwordlength=3, 
                extra_stopwords=[]):
    """
    Process text input
    
    Arguments:
    text -- text block to count
    
    Returns:
    Word count dictionary { word: count }
    """
    wordcount={}
    # quick and dirty text cleanup
    text = text.replace('-', ' ').replace('\'s', '').replace('\r\n', ' ').replace('\r', ' ').replace('\n', ' ').replace('\' ', ' ').replace(' \'', ' ')
    text = ''.join([c for c in text if c.isalpha() or c in ' \''])
    # split text and count words
    for word in text.split():
        word = word.strip()
        if word not in stoplist and word not in extra_stopwords and len(word) >= minwordlength:
            wordcount[word] = wordcount.get(word, 0) + 1
    return wordcount
        
def load_csv(file):
    """
    Quick and dirty CSV file load.  Assumes words are in first column and values in second
    
    Arguments:
    file -- name of CSV file
    
    Returns:
    Dictionary containing words and values { word : value }
    """
    data = {}
    try:
        with open(file, 'r+') as f:
            rdr = reader(f)
            for row in rdr:
                data[row[0]] = int(row[1])
        return data
    except IOError as error:
        print (error)
        return None



from pkg_resources import resource_filename

mask_ellipse = resource_filename('ajnwordcloud', '/mask_image/circle.png')
mask_roundrect = resource_filename('ajnwordcloud', '/mask_image/roundrect.png')
mask_speech = resource_filename('ajnwordcloud', '/mask_image/speech.png')
mask_cloud = resource_filename('ajnwordcloud', '/mask_image/cloud.png')
mask_star5 = resource_filename('ajnwordcloud', '/mask_image/star5.png')
mask_star6 = resource_filename('ajnwordcloud', '/mask_image/star6.png')
mask_thought = resource_filename('ajnwordcloud', '/mask_image/thought.png')
mask_heart = resource_filename('ajnwordcloud', '/mask_image/heart.png')
mask_xmas_tree = resource_filename('ajnwordcloud', '/mask_image/tree.png')
mask_usa = resource_filename('ajnwordcloud', '/mask_image/usa.png')
mask_uk = resource_filename('ajnwordcloud', '/mask_image/uk.png')
mask_alien = resource_filename('ajnwordcloud', '/mask_image/alien.png')


from pkg_resources import resource_filename

font_xkcd = resource_filename('ajnwordcloud', '/font/xkcd.otf')
font_mouse_memoirs = resource_filename('ajnwordcloud', '/font/MouseMemoirs.ttf')
font_chunkfive = resource_filename('ajnwordcloud', '/font/Chunkfive.otf')


import numpy as np
from PIL import Image, ImageDraw, ImageFont
from random import random, choice
from math import log, ceil
from operator import itemgetter
from . import ii_search
from .fonts import font_xkcd

def __generate_ii(im):
    """
    Create integral image array
    
    Arguments:
    im -- "L" mode PIL image
    
    Returns:
    2D numpy array of integral image
    """
    im_arr = np.fromstring(im.tobytes(), dtype=np.uint8)
    im_arr = im_arr.reshape((im.size[1], im.size[0]))
    return im_arr.cumsum(axis=0).cumsum(axis=1).astype(np.uint32)

def __scale_words(data, 
                  scale, 
                  maxwords, 
                  uppercase):
    """
    Calculate font sizes for each word using specified scaling method
        
    Arguments:
    data -- dictionary of word counts/values
    scale -- int value as base size for log scale
    maxwords -- maximum number of words used to create cloud
    uppercase -- convert words to uppercase
    
    Returns:
    List of word/font size tuples sorted in descending order
    """
    # scale values
    vocab = { k.upper() if uppercase else k: int(ceil(log(v) * scale)) for k, v in data.items() }
    # return provided max words from sorted list
    return sorted(vocab.items(), key=itemgetter(1), reverse=True)[0:maxwords]
    
def wordcloud(words, 
              fontname=font_xkcd, 
              cloudsize=(400, 400), 
              maxwords=500, 
              uppercase=False, 
              scale=12, 
              minfontsize=7, 
              colours=['black'], 
              background='white', 
              step=1, 
              fontstep=1, 
              rotated=0.5, 
              mask=None):
    """
    Generate the word cloud image
        
    Arguments:
    words -- dictionary of word counts
    fontname -- name of truetype font to use
    cloudsize -- size of wordcloud image (default: (400, 400)
    maxwords -- maximum number of words to use (default: 500)
    uppercase -- whether to change words to upper case (default: False)
    scale -- scaling factor for font sizes (default: 16)
    minfontsize -- minimum font size (default: 7)
    colours -- list of colour names/hex values to use, see colours.py for built in schemes (default: multi)
    background -- background colour to use (default: None)
    step -- size of step when scanning image for available areas (default: 1)
    fontstep -- size of font reduction when word doesn't fit (default: 1)
    rotated -- proportion of words to rotate when placed (default: 0.5) 
    mask -- black and white image to define shape/area to use for cloud, will be resized to cloudsize (default: None)
    
    Returns:
    im - PIL image of completed cloud
    """
    # scale font sizes and sort words
    words = __scale_words(words, scale, maxwords, uppercase)
    # create main image
    im = Image.new("RGBA", cloudsize, background)
    # open mask image or create blank b/w image
    if mask:
        im1bit = Image.open(mask).convert("L").resize(cloudsize, Image.NEAREST) 
    else:
        im1bit = Image.new("L", cloudsize, 0)
    #initial integral image
    ii = __generate_ii(im1bit)
    # arbitrary large font size for initial check
    fontsize = 5000
    for word, fontsize_orig in words[0:maxwords]:
        # check so font size never increases
        if fontsize_orig < fontsize:
            fontsize = fontsize_orig
        # handle word rotation
        rotate=False
        if random() < rotated:
            rotate = True
        while True:
            # get font at current size and measure
            font = ImageFont.truetype(fontname, fontsize)
            size = ImageDraw.Draw(im).textsize(word, font=font)
            searchsize = (size[1], size[0]) if rotate else size
            # find possible spots
            if step > 1:
                spots = ii_search.ii_search(ii, searchsize[0], searchsize[1], step=step)
            else:
                spots = ii_search.ii_search_nostep(ii, searchsize[0], searchsize[1])
            if len(spots) > 0:
                # spots available so create text patch and place randomly
                im1 = Image.new("RGB", (size[0] + 16, size[1] + 16), background)
                alpha = Image.new("L", im1.size, 0)
                ImageDraw.Draw(im1).text((8, 8), word, font=font, fill=choice(colours))
                ImageDraw.Draw(alpha).text((8, 8), word, font=font, fill='white')
                spot = choice(spots)
                if rotate:
                    im1 = im1.rotate(90, expand=1)
                    alpha = alpha.rotate(90, expand=1)
                im.paste(im1, (spot[1]-8, spot[0]-8), alpha)
                im1bit.paste(alpha, (spot[1]-8, spot[0]-8), alpha)
                # regenerate the integral image and move to next word
                ii = __generate_ii(im1bit)
                break
            else:
                # no spots, decrease font size and try again
                fontsize -= fontstep
            # return when we hit the min font size
            if fontsize <= minfontsize:
                return im
    # return when we run out of words
    return im


from .core import wordcloud
from .data import count_words, load_csv, load_txt
from .colours import red, green, blue, yellow, orange, pink, brown, white, gray, multi
from .fonts import font_xkcd, font_mouse_memoirs, font_chunkfive
from .mask_images import mask_ellipse, mask_roundrect, mask_cloud, mask_thought, mask_speech, mask_star5, mask_star6, mask_xmas_tree, mask_usa, mask_heart, mask_alien 
           
__version__ = '0.2.3'


from setuptools import setup, find_packages
from setuptools.extension import Extension

setup(name='ajnwordcloud',
      version='0.2.3',
      description='simple wordcloud generator',
      url='http://github.com/voiceoftreason/ajnwordcloud',
      author='Andy Newton',
      author_email='voiceoftreason@me.com',
      license='MIT',
      packages=find_packages(),
      install_requires=['pillow', 'numpy>=1.6.1'],
      ext_modules=[Extension("ajnwordcloud.ii_search",
                           ["ajnwordcloud/ii_search.c"],include_dirs=['.'])],
      include_package_data=True,
      zip_safe=False)